#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Ejemplo para graficar un histograma y analizar datos

"""

# Importamos librerías estándares

from numpy import *
import matplotlib.pyplot as plt

#%% Definimos funciones auxiliares

# Función de una gaussiana
def gaussiana(x,mu,sigma):
    """
    Devuelve el valor de una gausiana de valor medio mu 
    con desviacion estandar sigma para cada valor de x
    """
    
    # Este paso es sólo para facilitar el cálculo como vector
    x = array(x)
    
    return exp( -(x-mu)**2/(2*sigma**2) ) / (sigma * sqrt(2*pi))



#%% Histograma con todas sus partes


datos = genfromtxt('numeros_aleatoreos.txt')



# Grafico **********************
plt.figure()

# histograma de densidad
datos_histograma = plt.hist( datos , 20, density=True , alpha=0.7, label='datos')
# datos_histograma = plt.hist( datos , 20, density=True , histtype='step' , lw=3, color='C0')

valores, limites, _ = datos_histograma

# valor medio
media = datos.mean()
plt.axvline( media , lw=2 , alpha=0.8 , color='C1' , label='media')

error_media = std(datos)/sqrt(len(datos))
plt.errorbar( media, valores.max()/2 ,xerr=error_media, color='C1', ms=5 , capsize=5, fmt='o' )

# # obtengo posicion del maximo en el vector valores
# Imax = valores.argmax()
# moda = mean(  limites[Imax:Imax+2] )
# plt.axvline( moda , lw=2 , alpha=0.8 , color='C2' , label='moda')


# # Mediana
# N = len(datos)//2
# mediana = mean( sorted(datos)[N:N+2] )
# plt.axvline( mediana , lw=2 , alpha=0.8 , color='C4' , label='mediana')


# Simulación Gaussiana
xx = linspace(0,16,500)
yy = gaussiana( xx , mu=datos.mean(), sigma=datos.std() )

plt.plot( xx, yy, color='C3', lw=3 , label='modelo gaussiano')



# Leyendas
plt.legend()

# Etiquetas y grid
plt.xlabel("Valor")
plt.ylabel("Densidad de probabilidad")
plt.title(f'Histograma de {len(datos)} dados')
plt.grid(ls=':')



# plt.axhline( valores.max()/2 , ls='--', color='gray', lw=2 )
# puntos = array( plt.ginput(2) )
# print( 'El FWHM es:', diff(puntos[:,0])[0] )




#%% Histograma avanzado demostrativo


datos = genfromtxt('numeros_aleatoreos.txt')

fig, axx = plt.subplots(2,1, figsize=(12,8),  constrained_layout=True, sharex=True,
                        gridspec_kw=dict(height_ratios=[1,0.3]))


ax=axx[0]

datos_histograma = ax.hist( datos , 20, density=True , alpha=0.7, label='datos')
# datos_histograma = plt.hist( datos , 20, density=True , histtype='step' , lw=3, color='C0')

valores, limites, _ = datos_histograma

# valor medio
media = datos.mean()
ax.axvline( media , lw=2  , color='C1' , label='media')

error_media = std(datos)/sqrt(len(datos))
ax.errorbar( media, valores.max()/2 ,xerr=error_media, color='C1', ms=5 , capsize=5, fmt='o' )

ax.set_ylabel("Densidad de probabilidad")



ax=axx[1]


ax.plot( datos , arange(len(datos)), '.' )

ax.set_xticks( limites )

ax.set_xlabel(r'$\Delta f$ [Hz]')
ax.set_ylabel(r'[mV]')



ax.set_xlabel("Valor")
ax.set_ylabel("# de medición")

fig.suptitle(f'Histograma de {len(datos)} dados')

for ax in axx:
    ax.grid(b=True,linestyle=':',color='lightgray')
