#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Ejemplo de Teorema central del límite usando sumas de dados

En este ejemplo se simulan la suma de N dados en M tiradas y se analizan
cómo cambia la distribución de probabilidad

"""


# Importamos librerías estándares

from numpy import *
import matplotlib.pyplot as plt

#%% Definimos funciones auxiliares

# Función de una gaussiana
def gaussiana(x,mu,sigma):
    """
    Devuelve el valor de una gausiana de valor medio mu 
    con desviacion estandar sigma para cada valor de x
    """
    
    # Este paso es sólo para facilitar el cálculo como vector
    x = array(x)
    
    return exp( -(x-mu)**2/(2*sigma**2) ) / (sigma * sqrt(2*pi))



#%% sumar N dados M veces

# Parámetros del problema
N = 1
M = 600

# Reinicio el generador de números aleatoreos
random.seed(0)

# Simulación *****************************
num_dados   = N
num_tiradas = M

# Lista para acumular los resultados
resultados = []

for _ in range(num_tiradas):
    resultado = sum( random.randint(1,7,size=num_dados) )
    
    resultados.append( resultado )
    
# Convierto la lista a array por si quiero hacer cuentas
resultados = array(  resultados  )



# Grafico **********************
plt.figure()

# histograma de densidad
plt.hist( resultados , arange(resultados.min()-0.5,resultados.max()+0.6) , 
         density=True , rwidth=0.9)

# Línea vertical de valor medio
plt.axvline( resultados.mean() , color='C1' , lw=3 )

# Grafico una gausiana con el mismo sigma y mu
xx = linspace( N-3 , N*6+3 , 1000)
plt.plot( xx, gaussiana(xx, resultados.mean(), resultados.std() ) )

# Etiquetas y grid
plt.xlabel("Valor")
plt.ylabel("Densidad de probabilidad")
plt.title(f'Resultado de sumar {N} dados (realizado {M} veces)')
plt.grid(ls=':')




